/* Copyright (c) 1998, MpegTV LLC */
#ifndef _voh_api_h
#define _voh_api_h
#ifndef lint
static const char rcsid_voh_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/voh_api.h,v 1.112 1999/06/13 09:35:57 tristan Exp $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

/* VOH = Video Output Handler */

#include "mtvp/misc.h"

#include "mtvp/dith_im.h"
#include "mtvp/dither_api.h"
#include "mtvp/msg.h"

#ifdef USE_X
#include "mtvp/voh_x_api.h"
#endif

/*--------------------------------------------------------------------------+
| Configuration																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
#define VOH_VERSION	4

#define VOH_X_HANDLER_ID		255
#define VOH_SDL_HANDLER_ID		254
#define VOH_NULL_HANDLER_ID		253

/* max depth to match in TrueColor visuals */
#define VOH_FULL_COLOR_MAX_DEPTH_DEFAULT	32

#define VOH_RED		0
#define VOH_GREEN	1
#define VOH_BLUE	2
#define VOH_RGB		3	/* size of arrays indexed by RED, GREEN, BLUE */

/* voh_control_flags */
#define VOH_CONTROL_PEL_ASPECT_RATIO			0x0001

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/

typedef struct {
	u_char	r, g, b;
} VOH_CLUT_ENTRY;

typedef struct {
    u_long		mask;
    u_char		number_of_bits_set;
    u_char		free_bits_at_bottom;
} VOH_RGB_MASK;

/* generic window id */
typedef void *VOH_WINDOW;

typedef struct voh_resources VOH_RESOURCES;

struct voh_resources {
	MSG_STRUCT *msg;
	const void *z;
	int handler_id;
	int (*feedback)(const VOH_RESOURCES *voh_resources, const void *data, size_t size);
	int (*dith_class_init)(const VOH_RESOURCES *voh_resources);
	void (*dith_class_free)(const VOH_RESOURCES *voh_resources);
	u_int (*dith_get_clut)(const VOH_RESOURCES *voh_resources, u_int max_colors, VOH_CLUT_ENTRY *clut);
	void (*disp_refresh)(const VOH_RESOURCES *voh_resources);
	void (*notify_fd)(const VOH_RESOURCES *voh_resources, int fd);
};

typedef struct {
	u_int horizontal;	/* mpeg Horizontal_size, from which Frame_width is derived */
	u_int vertical;		/* mpeg Vertical_size, from which Frame_height is derived */
	u_int frame_width;		/* reconstructed frame width in pels (always multiple of MB) */
	u_int frame_height;		/* reconstructed frame height in pels (always multiple of MB) */
} VOH_SIZE;

typedef struct {
	double  pel_aspect_ratio;	/* 0 means free i.e. based on window geometry */
} VOH_CONTROL;

/* struct voh_data is private to each VOH handler */
struct voh_data;

/* block of data received by application with A2G_COM_VOH_FEEDBACK */
typedef struct {
	int handler_id;
	char data[1];	/* this is a dummy declaration. in fact data may be any size */
} VOH_FEEDBACK;

/* block of data sent by application with COM_VOH_CNTL */
typedef struct {
	int handler_id;
	char data[1];	/* this is a dummy declaration. in fact data may be any size */
} VOH_CNTL;


typedef struct {
	int	VOH_version;	/* handler must set this to SIH_VERSION */

	struct voh_data *(*VOH_init)(const VOH_RESOURCES *voh_resources, VOH_DITHERED_IMAGE *voh_dithered_image);
	void (*VOH_free)(struct voh_data *voh_data);

	int (*VOH_window_create)(struct voh_data *voh_data, DITHER_TYPE dither_type, const VOH_SIZE *voh_size, VOH_WINDOW existing_window, const VOH_CONTROL *voh_control, u_long voh_control_flags);
	void (*VOH_window_destroy)(struct voh_data *voh_data);

	int (*VOH_resolve_dither)(struct voh_data *voh_data, VOH_WINDOW	existing_window, Bool *dither_ok, DITHER_TYPE *prefered_type_ptr);

	int (*VOH_read_and_handle_all_pending_events)(struct voh_data *voh_data, Bool must_handle_at_least_one_event);

	int (*VOH_dithered_image_alloc)(struct voh_data *voh_data);
	void (*VOH_dithered_image_free)(struct voh_data *voh_data);
	void (*VOH_dithered_image_show)(struct voh_data *voh_data);
	Bool (*VOH_dithered_image_lock)(struct voh_data *voh_data);
	void (*VOH_dithered_image_unlock)(struct voh_data *voh_data);

	int (*VOH_set_clut)(struct voh_data *voh_data);
	void (*VOH_free_clut)(struct voh_data *voh_data);

	const u_char *(*VOH_get_clut_pixel_values)(struct voh_data *voh_data);
	void (*VOH_get_rgb_masks)(struct voh_data *voh_data, VOH_RGB_MASK *masks);

	int (*VOH_set_control)(struct voh_data *voh_data, VOH_CONTROL *voh_control, u_long voh_control_flags);
	int (*VOH_cntl)(struct voh_data *voh_data, const VOH_CNTL *ptr, u_int size);
} VOH_EXPOSED;

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

extern int _init(void);
extern int _fini(void);
extern int	VOH_register(VOH_EXPOSED *exposed, int version);

#ifdef __cplusplus
}
#endif

#endif /* _voh_api_h */
