/* Copyright (c) 1998, MpegTV LLC */
#ifndef _voh_x_api_h
#define _voh_x_api_h
#ifndef lint
static const char rcsid_voh_x_api_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/voh_x_api.h,v 1.104 1999/05/01 08:13:18 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

/* VOH = Video Output Handler */

#include "mtvp/misc.h"

#include <X11/Xlib.h>

/*--------------------------------------------------------------------------+
| Configuration																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/

/* use root window */
#define VOH_X_WINDOW_ROOT	((VOH_WINDOW)(-1))

/* create new window (default) */
#define VOH_X_WINDOW_NONE	((VOH_WINDOW)0)

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/


/* Specific VOH_FEEDBACK data from the X Video Output Handler */
typedef struct {

	/* type of X-specific message */
	enum {
		VOH_X_FEEDBACK_TYPE_NO_SHARED_MEMORY = 0,	/* no associated data */
		VOH_X_FEEDBACK_TYPE_EVENT = 1,				/* associated data is XEvent */
		VOH_X_FEEDBACK_TYPE_PRIVATE_CLUT = 2,		/* no associated data */
	} type;	

	/* associated data */
	union {
		XEvent	event;	
	} data;
	
} VOH_X_FEEDBACK_DATA;

/* Specific A2G_COM_VOH_X_FEEDBACK data from the X Video Output Handler */
typedef struct {
	int handler_id;		/* VOH_X_HANDLER */
	VOH_X_FEEDBACK_DATA data_x;
} VOH_X_FEEDBACK;


/* type of video layout */
typedef enum video_layout {
	VOH_X_VIDEO_LAYOUT_CENTERED = 0,
	VOH_X_VIDEO_LAYOUT_FILL_WINDOW = 1,
	VOH_X_VIDEO_LAYOUT_TOP_LEFT = 2
} VOH_X_VIDEO_LAYOUT;

struct window_position {
	int		x;	/* window horizontal position */
	int		y;	/* window vertical position */
};

struct scale {
	double  horizontal;
	double  vertical;
};

/* Specific VOH_CNTL for the X Video Output Handler */
typedef struct {

	/* VOH_CNTL structure must ALWAYS start with int handler_id */
	int handler_id;		/* VOH_X_HANDLER */

	/* type of X-specific message */
	enum {
		VOH_X_CNTL_TYPE_SET_EVENT_MASK = 0,
		VOH_X_CNTL_TYPE_USE_PRIVATE_CLUT = 1,
		VOH_X_CNTL_TYPE_RAISE_WINDOW = 2,
		VOH_X_CNTL_TYPE_SET_VIDEO_LAYOUT = 3,
		VOH_X_CNTL_TYPE_SET_ICON_NAME = 4,
		VOH_X_CNTL_TYPE_SET_WINDOW_NAME = 5,
		VOH_X_CNTL_TYPE_SET_WINDOW_POSITION = 6,
		VOH_X_CNTL_TYPE_SET_SCALE = 7,
#if 0 /* Tristan Savatier - Wed May 5 02:30:21 PDT 1999 */
//		VOH_X_CNTL_TYPE_USE_EXISTING_WINDOW = 8,
#endif /* 0 - Tristan Savatier - Wed May 5 02:30:21 PDT 1999 */
	} type;	

	/* associated data */
	union {
		long				event_mask;
		Bool				use_private_clut;	
		VOH_X_VIDEO_LAYOUT	video_layout;	
		char				icon_name[256];	
		char				window_name[256];	
		struct scale 		scale;
		struct window_position window_position;
	} data;
	
} VOH_X_CNTL;


#endif /* _voh_x_api_h */
