#ifndef lint
static const char rcsid[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/voh_x.c,v 1.246 1999/03/20 12:01:19 tristan Exp tristan $";
static const char copyright[] = "Copyright (c) 1997, MpegTV LLC";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/misc.h"
#include "mtvp/dith_im.h"
#include "mtvp/msg_api.h"
#include "mtvp/voh_api.h"

#include "SDL.h"
#include "SDL_syswm.h"

/*--------------------------------------------------------------------------+
| Configuration
+--------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------+
| Macros
+--------------------------------------------------------------------------*/
#undef MSG_PTR
#define MSG_PTR	(((struct voh_data *)voh_data)->voh_resources->msg)


/*--------------------------------------------------------------------------+
| Data types
+--------------------------------------------------------------------------*/


typedef enum {
	SDL_BPP_MODE_BEST = 0,
	SDL_BPP_MODE_16 = 1,
	SDL_BPP_MODE_24 = 2,
	SDL_BPP_MODE_32 = 3
} SDL_BPP_MODE;


struct voh_data {
	SDL_BPP_MODE sdl_bpp_mode;		/* bpp mode (best or specified bpp) */
	int		sdl_bpp;				/* bits per pixel */
	int		sdl_video_flags;		/* SDL video flags */
	Bool	sdl_flip;				/* if True use SDL_Flip */
	SDL_Surface *sdl_screen;		/* NULL or ptr to SDL screen */
	
	VOH_DITHERED_IMAGE *voh_dithered_image;
	const VOH_RESOURCES *voh_resources;
	VOH_CONTROL 	voh_control;
};


/*--------------------------------------------------------------------------+
| VOH_dithered_image_show
+--------------------------------------------------------------------------*/
static void
VOH_dithered_image_show(
	struct voh_data *voh_data
)
{
	u_int	x;
	u_int	y;

	MSG(MSG_X, ("X: showing dithered image"));

  	if (voh_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}
	
	if (voh_data->sdl_flip) {
		/* printf("SDL_Flip...\n"); */
		if (SDL_Flip(voh_data->sdl_screen) < 0) {
			abort_now();
		}
	} else {
		if (SDL_MUSTLOCK(voh_data->sdl_screen)) {
			/* printf("SDL_UnlockSurface...\n"); */
			SDL_UnlockSurface(voh_data->sdl_screen);
			voh_data->voh_dithered_image->array = NULL;
		}
	}
}


/*--------------------------------------------------------------------------+
| VOH_window_create
+--------------------------------------------------------------------------*/
static int
VOH_window_create (
	struct voh_data *voh_data,
	DITHER_TYPE dither_type,
	const VOH_SIZE *voh_size,
	VOH_WINDOW	existing_window,
	VOH_CONTROL *voh_control,
	u_long	flags
)
{
	int     screen;
	const SDL_VideoInfo *video_info;

  	if (voh_data->sdl_screen != NULL) {
		abort_now(); /* should never happen */
	}

	if (flags & VOH_CONTROL_PEL_ASPECT_RATIO) {
		/* aspect ratio handling not implemented */
	}
			
	if (dither_type != DITHER_COLOR) {
		printf("mtvp: fullscreen can only be used with full-color video\n");
		printf("mtvp: restart the X server with 16-bit, 24-bit or 32-bit depth\n");
		printf("mtvp: with: startx -- -bpp <depth> [e.g. startx -- -bpp 16]\n");				printf("mtvp: and make sure that video mode is set to full-color\n");
		goto sdl_failed;
	}

	/* Inizialize the SDL library */
	MSG(MSG_X, ("X: SDL: calling SDL_Init"));
	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
		printf("mtvp: Couldn't initialize SDL: %s\n", SDL_GetError());
		printf("mtvp: Make sure you have /usr/X11R6/lib/libSDLx11.so.0.9\n");
		printf("mtvp: This library (SDL) can be downloaded from the MpegTV FTP site\n");
		printf("mtvp: ftp://ftp.mpegtv.com/pub/mpeg/mpegtv/player\n");
		goto sdl_failed;
	}
	MSG(MSG_X, ("X: SDL: SDL_Init succeeded"));
	SDL_EventState(SDL_ACTIVEEVENT, SDL_IGNORE);
	SDL_EventState(SDL_MOUSEMOTION, SDL_IGNORE);
	video_info = SDL_GetVideoInfo();

	MSG(MSG_X, ("X: SDL: best video mode is %u-bit surface", video_info->vfmt->BitsPerPixel));
	
	MSG(MSG_X, ("X: sdl_bpp_mode = %u", voh_data->sdl_bpp_mode));
	
	switch (voh_data->sdl_bpp_mode) {
		case SDL_BPP_MODE_BEST:
			voh_data->sdl_bpp = video_info->vfmt->BitsPerPixel;
			break;
		case SDL_BPP_MODE_16:	voh_data->sdl_bpp = 16; break;
		case SDL_BPP_MODE_24:	voh_data->sdl_bpp = 24; break;
		case SDL_BPP_MODE_32:	voh_data->sdl_bpp = 32; break;
		default:				abort_now(); break;
	}

	MSG(MSG_X, ("X: SDL: will use a %u-bit surface", voh_data->sdl_bpp));

	if (voh_data->sdl_bpp != 16 && voh_data->sdl_bpp != 24 && voh_data->sdl_bpp != 32) {
		MSG(MSG_X, ("X: SDL: unsupported BitsPerPixel %u", voh_data->sdl_bpp));
		MSG(MSG_X, ("X: SDL: calling SDL_Quit"));
		SDL_Quit();
		printf("mtvp: cannot use fullscreen when the X server depth is 8-bit\n");
		printf("mtvp: restart the X server with 16-bit, 24-bit or 32-bit depth\n");
		printf("mtvp: with: startx -- -bpp <depth> [e.g. startx -- -bpp 16]\n");
		goto sdl_failed;
	}
			
	MSG(MSG_X, ("X: SDL: calling SDL_SetVideoMode"));
	voh_data->sdl_screen = SDL_SetVideoMode(voh_size->frame_width, voh_size->frame_height, voh_data->sdl_bpp, voh_data->sdl_video_flags);

	if (voh_data->sdl_screen == NULL) {
		MSG(MSG_X, ("X: SDL: Couldn't initialize video mode: %s", SDL_GetError()));
		printf("mtvp: Couldn't set SDL video mode: %s\n", SDL_GetError());
		printf("mtvp: make sure that XFree86 is configured to support at least one\n");
		printf("mtvp: resolution equal or larger than the video size of this stream.\n");
		goto sdl_failed;
	}
	MSG(MSG_X, ("X: SDL: SDL_SetVideoMode succeeded"));

	assert(voh_data->sdl_screen->format->BitsPerPixel == 16
		|| voh_data->sdl_screen->format->BitsPerPixel == 24
		|| voh_data->sdl_screen->format->BitsPerPixel == 32);

	if (voh_data->sdl_video_flags & SDL_FULLSCREEN) {
		if (voh_data->sdl_screen->flags & SDL_FULLSCREEN) {
			MSG(MSG_X, ("X: SDL: screen supports full screen"));
		} else {
			uid_t	euid = geteuid();

			MSG(MSG_X, ("X: SDL: screen doesn't support full screen"));
			printf("mtvp: cannot use fullscreen\n");
			
#if 0 /* Tristan Savatier - Mon Dec 14 03:39:17 PST 1998 */
//					printf("euid = %d\n", (int)euid);
#endif /* 0 - Tristan Savatier - Mon Dec 14 03:39:17 PST 1998 */
			if (euid != 0) {
				printf("mtvp: You must be root (or mtvp must be set-uid root) to use fullscreen\n");
				printf("mtvp: For set-uid root, become root and use the following commands:\n");
				printf("mtvp: $ chown root mtvp; chmod +s mtvp\n");
				printf("mtvp: Then restart the program and try again...\n");
				printf("mtvp: Note: set-uid programs may create security holes\n");
			}
			printf("mtvp: For details, read the README-Fullscreen file.\n");
#if 0 /* Tristan Savatier - Sun Mar 21 23:57:26 PST 1999 */
//			goto sdl_failed;
#endif /* 0 - Tristan Savatier - Sun Mar 21 23:57:26 PST 1999 */
		}
	}
	if (voh_data->sdl_screen->flags & SDL_HWSURFACE) {
		MSG(MSG_X, ("X: SDL: screen supports hardware surface"));
		if (voh_data->sdl_screen->flags & SDL_DOUBLEBUF) {
			MSG(MSG_X, ("X: SDL: screen supports double buffering"));
			voh_data->sdl_flip = True;
		} else {
			MSG(MSG_X, ("X: SDL: screen doesn't support double buffering"));
			voh_data->sdl_flip = False;
		}
	} else {
		MSG(MSG_X, ("X: SDL: screen doesn't support hardware surface"));
		voh_data->sdl_flip = True;
	}
		
#if 0 /* Tristan Savatier - Sun Mar 21 23:08:49 PST 1999 */
//	voh_data->depth = voh_data->sdl_screen->format->BitsPerPixel;
#endif /* 0 - Tristan Savatier - Sun Mar 21 23:08:50 PST 1999 */
				
#if 0 /* Tristan Savatier - Mon Dec 14 03:31:38 PST 1998 */
//			printf("voh_data->sdl_fd = %d\n", voh_data->sdl_fd);
//			printf("voh_data->sdl_screen->flags = 0x%08x\n", voh_data->sdl_screen->flags);
//			printf("SDL_HWSURFACE = %d\n", !!(voh_data->sdl_screen->flags & SDL_HWSURFACE));
//			printf("SDL_THREADSAFE = %d\n", !!(voh_data->sdl_screen->flags & SDL_THREADSAFE));
//			printf("SDL_ANYFORMAT = %d\n", !!(voh_data->sdl_screen->flags & SDL_ANYFORMAT));
//			printf("SDL_HWPALETTE = %d\n", !!(voh_data->sdl_screen->flags & SDL_HWPALETTE));
//			printf("SDL_DOUBLEBUF = %d\n", !!(voh_data->sdl_screen->flags & SDL_DOUBLEBUF));
//			printf("SDL_FULLSCREEN = %d\n", !!(voh_data->sdl_screen->flags & SDL_FULLSCREEN));
//			printf("SDL_HWACCEL = %d\n", !!(voh_data->sdl_screen->flags & SDL_HWACCEL));
//			printf("voh_data->sdl_screen->format->BitsPerPixel = %d\n", voh_data->sdl_screen->format->BitsPerPixel);
//			printf("voh_data->sdl_screen->format->BytesPerPixel = %d\n", voh_data->sdl_screen->format->BytesPerPixel);
//			printf("voh_data->sdl_screen->format->Rloss = %d\n", voh_data->sdl_screen->format->Rloss);
//			printf("voh_data->sdl_screen->format->Gloss = %d\n", voh_data->sdl_screen->format->Gloss);
//			printf("voh_data->sdl_screen->format->Bloss = %d\n", voh_data->sdl_screen->format->Bloss);
//			printf("voh_data->sdl_screen->format->Aloss = %d\n", voh_data->sdl_screen->format->Aloss);
//			printf("voh_data->sdl_screen->format->Rshift = %d\n", voh_data->sdl_screen->format->Rshift);
//			printf("voh_data->sdl_screen->format->Gshift = %d\n", voh_data->sdl_screen->format->Gshift);
//			printf("voh_data->sdl_screen->format->Bshift = %d\n", voh_data->sdl_screen->format->Bshift);
//			printf("voh_data->sdl_screen->format->Ashift = %d\n", voh_data->sdl_screen->format->Ashift);
//			printf("voh_data->sdl_screen->format->Rmask = %d\n", voh_data->sdl_screen->format->Rmask);
//			printf("voh_data->sdl_screen->format->Gmask = %d\n", voh_data->sdl_screen->format->Gmask);
//			printf("voh_data->sdl_screen->format->Bmask = %d\n", voh_data->sdl_screen->format->Bmask);
//			printf("voh_data->sdl_screen->format->Amask = %d\n", voh_data->sdl_screen->format->Amask);
//			printf("voh_data->sdl_screen->w = %d\n", voh_data->sdl_screen->w);
//			printf("voh_data->sdl_screen->h = %d\n", voh_data->sdl_screen->h);
//			printf("voh_data->sdl_screen->pitch = %d\n", voh_data->sdl_screen->pitch);
#endif /* 0 - Tristan Savatier - Mon Dec 14 03:31:38 PST 1998 */

	{
		SDL_SysWMinfo info;

		SDL_VERSION(&info.version);
		if (SDL_GetWMInfo(&info) < 0) {
			MSG(MSG_X, ("X: SDL: SDL_GetWMInfo failed: %s", SDL_GetError()));
			goto sdl_failed;
		}
		/* connextion number (fd) to X display used by SDL */
		(*voh_data->voh_resources->notify_fd)(voh_data->voh_resources, ConnectionNumber(info.display));
	}


	return 0;

sdl_failed:
	MSG(MSG_X, ("X: SDL: cannot use SDL"));
	MSG(MSG_X, ("X: SDL: calling SDL_Quit"));
	SDL_Quit();
	voh_data->sdl_screen = NULL;
	errno = EINVAL;
	return -1;
}


/*--------------------------------------------------------------------------+
| VOH_window_destroy
+--------------------------------------------------------------------------*/
static int
VOH_window_destroy (
	struct voh_data *voh_data
)
{
  	if (voh_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}
	
	MSG(MSG_X, ("X: SDL: calling SDL_Quit"));
  	SDL_Quit();
	voh_data->sdl_screen = NULL;

	return 0;
}

/*--------------------------------------------------------------------------+
| VOH_dithered_image_free
+--------------------------------------------------------------------------*/
static void
VOH_dithered_image_free (
	struct voh_data *voh_data
)
{
	/* nothing to do in this case */
}


/*--------------------------------------------------------------------------+
| VOH_dithered_image_alloc
+--------------------------------------------------------------------------*/
static int
VOH_dithered_image_alloc(
	struct voh_data *voh_data
)
{
	if (voh_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}
	voh_data->voh_dithered_image->array = voh_data->sdl_screen->pixels;
	voh_data->voh_dithered_image->bytes_per_pel = voh_data->sdl_screen->format->BytesPerPixel;
	voh_data->voh_dithered_image->width = voh_data->sdl_screen->w;
	voh_data->voh_dithered_image->pitch = voh_data->sdl_screen->pitch;
	voh_data->voh_dithered_image->height = voh_data->sdl_screen->h;

	return 0;
}



/*--------------------------------------------------------------------------+
| VOH_resolve_dither
+--------------------------------------------------------------------------*/
static int
VOH_resolve_dither(
	struct voh_data *voh_data,
	VOH_WINDOW	existing_window,
	Bool	*dither_ok,
	DITHER_TYPE *prefered_type_ptr
)
{
	/* In this example, we only handle full-color dither */
	
	dither_ok[DITHER_GRAY] = False;
	dither_ok[DITHER_ORDERED] = False;
	dither_ok[DITHER_COLOR] = True;
	
	*prefered_type_ptr = DITHER_COLOR;
	
	return 0;
}


/*--------------------------------------------------------------------------+
| VOH_read_and_handle_all_pending_events
+--------------------------------------------------------------------------*/
static int
VOH_read_and_handle_all_pending_events(
	struct voh_data *voh_data,
	Bool must_handle_at_least_one_event
)
{
	/*
	 * Return number of events processed, or -1 if an error occured.
	 */

	/* in this example, any event causes the application to exit */
	
  	if (voh_data->sdl_screen != NULL) {
  	
  		if (SDL_PollEvent(NULL) != 0) {
			printf("VOH_read_and_handle_all_pending_events: found event, exiting\n");
			SDL_Quit();
			exit(0);	/* you should not do that in a real application, of course */
  		}
  	}
  	return 0;
}


/*--------------------------------------------------------------------------+
| VOH_set_control
+--------------------------------------------------------------------------*/
static int
VOH_set_control(
	struct voh_data *voh_data,
	VOH_CONTROL *voh_control,
	u_long	flags
)
{
	if (flags & VOH_CONTROL_PEL_ASPECT_RATIO) {
		/* aspect ratio change not implemented */
	}

	return 0;
}


/*--------------------------------------------------------------------------+
| VOH_cntl
+--------------------------------------------------------------------------*/
static int
VOH_cntl(
	struct voh_data *voh_data,
	const void *data,
	u_int	size
)
{
	/*
	 * VOH_cntl is called if and only if the application sends
	 * the message COM_VOH_CNTL.  The parameter "data" is a pointer to
	 * an arbitrary chunk of data specified by the application.
	 * "size" is the size of the chunk of data.
	 */
	 
	/*
	 * In this example, the data block passed to VOH_cntl() is known to be
	 * a string, so we just print it to show that we received it.
	 */
	 
	printf("VOH_cntl: data = \"%s\", size = %u\n", (const char *)data, size);
	
	return 0;
}



/*--------------------------------------------------------------------------+
| VOH_get_cm_pixel_values
+--------------------------------------------------------------------------*/
static u_long *
VOH_get_cm_pixel_values(
	struct voh_data *voh_data
)
{
	/* not implemented since we only handle full-color windows */
	return NULL;
}

/*--------------------------------------------------------------------------+
| VOH_get_rgb_masks
+--------------------------------------------------------------------------*/
static void
VOH_get_rgb_masks(
	struct voh_data *voh_data,
    VOH_RGB_MASK *masks
)
{
  	if (voh_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}

	masks[VOH_RED].mask = voh_data->sdl_screen->format->Rmask;
	masks[VOH_GREEN].mask = voh_data->sdl_screen->format->Gmask;
	masks[VOH_BLUE].mask = voh_data->sdl_screen->format->Bmask;
}


/*--------------------------------------------------------------------------+
| VOH_free
+--------------------------------------------------------------------------*/
static void
VOH_free(
	struct voh_data *voh_data
)
{
	if (voh_data->sdl_screen != NULL) {
    	MSG(MSG_X, ("X: SDL: calling SDL_Quit"));
		SDL_Quit();
	}

    MSG(MSG_X, ("X: releasing voh_data memory"));
	bzero(voh_data, sizeof *voh_data);	/* helps to catch bugs */
	free(voh_data);
}


/*--------------------------------------------------------------------------+
| VOH_dithered_image_lock
+--------------------------------------------------------------------------*/
static Bool
VOH_dithered_image_lock(
	struct voh_data *voh_data
)
{
	/*
	 * Must return False if dithered image is displayed instantly when
	 * written into (i.e. hardware plane with no double buffer).
	 * Must return True if dithered image is displayed only after
	 * VOH_dithered_image_show has been called.
	 */
	
  	if (voh_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}

	if (SDL_MUSTLOCK(voh_data->sdl_screen)) {
		/* printf("SDL_LockSurface...\n"); */
		if (SDL_LockSurface(voh_data->sdl_screen) <0 ) {
			abort_now();
		}
		voh_data->voh_dithered_image->array = voh_data->sdl_screen->pixels;
	}	
	return voh_data->sdl_flip;
}

/*--------------------------------------------------------------------------+
| VOH_dithered_image_unlock
+--------------------------------------------------------------------------*/
static void
VOH_dithered_image_unlock(
	struct voh_data *voh_data
)
{
  	if (voh_data->sdl_screen == NULL) {
		abort_now(); /* should never happen */
	}
		
	if (!voh_data->sdl_flip && SDL_MUSTLOCK(voh_data->sdl_screen)) {
		/* printf("SDL_UnlockSurface...\n"); */
		SDL_UnlockSurface(voh_data->sdl_screen);
		voh_data->voh_dithered_image->array = NULL;
	}
}

/*--------------------------------------------------------------------------+
| VOH_init
+--------------------------------------------------------------------------*/
static struct voh_data *
VOH_init(
	const VOH_RESOURCES *voh_resources,
	VOH_DITHERED_IMAGE *voh_dithered_image
)
{
	struct voh_data *voh_data;
	
    voh_data = (struct voh_data *)calloc(1, sizeof(struct voh_data));
    
    if (voh_data == NULL) {
		MSG_ERROR (MSG_X, ("X: cannot allocate voh_data"));
		return NULL;
    }

    voh_data->voh_resources = voh_resources;
    voh_data->voh_dithered_image = voh_dithered_image;    

	voh_data->sdl_bpp_mode = SDL_BPP_MODE_BEST;
	voh_data->sdl_video_flags = SDL_HWPALETTE|SDL_FULLSCREEN|SDL_DOUBLEBUF|SDL_HWSURFACE;

	return voh_data;
}

/*--------------------------------------------------------------------------+
| VOH_set_clut
+--------------------------------------------------------------------------*/
static int
VOH_set_clut(
	struct voh_data *xinfo
)
{
	return 0;
}

/*--------------------------------------------------------------------------+
| VOH_free_clut
+--------------------------------------------------------------------------*/
static void
VOH_free_clut(
	struct voh_data *xinfo
)
{
}

/*--------------------------------------------------------------------------+
| VOH_get_clut_pixel_values
+--------------------------------------------------------------------------*/
static const u_char *
VOH_get_clut_pixel_values(
	struct voh_data *xinfo
)
{
	return NULL;
}

/*--------------------------------------------------------------------------+
| VOH_register
+--------------------------------------------------------------------------*/
int
VOH_register(VOH_EXPOSED *exposed, int version)
{
	if (version == VOH_VERSION) {

		exposed->VOH_version = VOH_VERSION;
		exposed->VOH_init = VOH_init;
		exposed->VOH_free = VOH_free;
		exposed->VOH_window_create = VOH_window_create;
		exposed->VOH_window_destroy = VOH_window_destroy;
		exposed->VOH_resolve_dither = VOH_resolve_dither;
		exposed->VOH_read_and_handle_all_pending_events = VOH_read_and_handle_all_pending_events;
		exposed->VOH_dithered_image_alloc = VOH_dithered_image_alloc;
		exposed->VOH_dithered_image_free = VOH_dithered_image_free;
		exposed->VOH_dithered_image_show = VOH_dithered_image_show;
		exposed->VOH_dithered_image_lock = VOH_dithered_image_lock;
		exposed->VOH_dithered_image_unlock = VOH_dithered_image_unlock;
		exposed->VOH_set_clut = VOH_set_clut;
		exposed->VOH_free_clut = VOH_free_clut;
		exposed->VOH_get_clut_pixel_values = VOH_get_clut_pixel_values;
		exposed->VOH_get_rgb_masks = VOH_get_rgb_masks;
		exposed->VOH_set_control = VOH_set_control;
		exposed->VOH_cntl = VOH_cntl;
	}
	return VOH_VERSION;
}


/* eof */
